#  ARTUR VINAS FERRAN
#  FIRST IMPLEMENTATION
#  ---------------------------------------------------------------------------------------------------------
#  ---------------------------------------------------------------------------------------------------------
import os
import argparse


def read_arguments():
    """
    Parse and return the command-line arguments for input and output file paths.

    This function defines the command-line arguments for processing input files and the output file/directory,
    and then parses these arguments to extract their values.

    Returns
    -------
    tuple of str
        A tuple containing:
        - input_path_1 (str): The path of the first input file.
        - input_path_2 (str): The path of the second input file.
        - output_path (str): The path of the output directory.
        - output_name (str or None): The name of the output file (optional).

    Examples
    --------
    >>> input_path_1, input_path_2, output_path, output_name = read_arguments()
    """
    parser = argparse.ArgumentParser(description='Process_files')

    # Define the inputs (-i and -g) and the output (-o)
    parser.add_argument("-i", "--input1", type=str, required=True, help="The path of the first input")
    parser.add_argument("-g", "--input2", type=str, required=True, help="The path of the second input")
    parser.add_argument("-o", "--output", type=str, required=True, help="The path of the output file")
    parser.add_argument("-n", "--doc_name", type=str, required=False, help="The name of the output file (optional)")

    args = parser.parse_args()

    # Assign the inputs and output given to different variables
    input_path_1 = args.input1
    input_path_2 = args.input2
    output_path = args.output
    output_name = args.doc_name

    return input_path_1, input_path_2, output_path, output_name


class Concatenator(object):
    def __init__(self, input_path_1, input_path_2, output_path, output_name):
        self.input_path_1 = input_path_1
        self.input_path_2 = input_path_2
        self.output_path = output_path
        self.output_name = output_name

    @staticmethod
    def check(input_path_1, input_path_2, output_path):
        """
        Confirm the existence of input files and output directory, and read the input files.

        Parameters
        ----------
        input_path_1 : str
            The file path for the first input document.
        input_path_2 : str
            The file path for the second input document.
        output_path : str
            The directory path where the output file will be saved.

        Returns
        -------
        tuple of str
            A tuple containing the content of the first and second input documents.

        Raises
        ------
        SystemExit
            If any of the input files or the output directory do not exist, the program will print an error message and
            stop.

        Examples
        --------
        >>> input_path_1 = "/path/to/first_document.txt"
        >>> input_path_2 = "/path/to/second_document.txt"
        >>> output_path = "/path/to/output_directory"
        >>> doc1, doc2 = check(input_path_1, input_path_2, output_path)
        """
        # Confirm that both input files exist
        if not os.path.exists(input_path_1):
            print("The file or directory from the first input (or both) does not exist")
            exit()
        if not os.path.exists(input_path_2):
            print("The file or directory from the second input (or both) does not exist")
            exit()

        # Confirm that the output directory exists
        if not os.path.isdir(output_path):
            print("The directory of the output file doesn't exist")
            exit()

        # Read both input files
        doc1 = open(input_path_1).read()
        doc2 = open(input_path_2).read()

        return doc1, doc2

    @staticmethod
    def compute(doc1, doc2, output_name, output_path):
        """
        Concatenate two documents and write the result to a new file.

        Parameters
        ----------
        doc1 : str
            The content of the first document to be concatenated.
        doc2 : str
            The content of the second document to be concatenated.
        output_name : str or None
            The name of the output file. If None, defaults to 'document3.txt'.
        output_path : str
            The directory path where the output file will be saved.

        Returns
        -------
        None
            This function does not return any value.

        Examples
        --------
        >>> doc1 = "This is the first document."
        >>> doc2 = "This is the second document."
        >>> output_name = "combined_document.txt"
        >>> output_path = "/path/to/directory"
        >>> compute(doc1, doc2, output_name, output_path)
        """
        # Determine the full path of the output file
        if output_name is None:
            output_file = os.path.join(output_path, "document3.txt")
        else:
            output_file = os.path.join(output_path, output_name)

        # Create a new doc and concatenate both files
        with open(output_file, "a") as doc3:
            doc3.write(doc1)
            doc3.write(doc2)

        return None

    def main(self):
        """
        Main function to process input arguments, validate files, and concatenate document contents.

        This function performs the following steps:
        1. Reads the input arguments (input files and output directory).
        2. Checks the existence of the input files and output directory.
        3. Concatenates the contents of the input files and writes the result to the output file.

        Parameters
        ----------
        self : object
            An instance of the class containing this `main` method. The class is expected to have
            `read_arguments`, `check`, and `compute` methods.

        Returns
        -------
        None
            This function does not return any value.

        Examples
        --------
        >>> processor = DocumentProcessor()
        >>> processor.main()
        """

        # Check that they exist
        doc1, doc2 = self.check(self.input_path_1, self.input_path_2, self.output_path)

        # Generate the concatenation file of the two initial input files
        self.compute(doc1, doc2, self.output_name, self.output_path)


def run():
    #  Read the inputs and output arguments
    input1, input2, output_path, output_name = read_arguments()
    #  Concatenate them using the class Concatenator
    concatenate = Concatenator(input1, input2, output_path, output_name)
    concatenate.main()


if __name__ == "__main__":
    run()
